#ifndef _CCAPTION_PAINTER_H_
#define _CCAPTION_PAINTER_H_

#include <windows.h>
#include "PseudoOSD.h"
#include "CaptionDllUtil.h"
#include <list>
#include <vector>

class COSDManager;

typedef struct _CAPTION_SHOWPROC_PARM {
	HWND hViewer;
	unsigned int waitTime;
	COSDManager * pOSDManager;
	std::vector<CAPTION_DATA> *pCaptions;
} CAPTION_SHOWPROC_PARM, *PCAPTION_SHOWPROC_PARM;

class COSDManager {

	typedef std::list<CPseudoOSD *>   OSD_LIST, *POSD_LIST;

	HWND					m_hParentView;

	CRITICAL_SECTION		m_csList;
	OSD_LIST				m_freeOSDList;
	OSD_LIST				m_inuseOSDList;

	CPseudoOSD *			GetOSD();
	void					MarkAsInUse(CPseudoOSD * pOSD);

public:

	typedef struct _OSD_DISPLAY_PARM {
		COLORREF			color;					// Text Color
		int					textHeight;				// Text Size
		LPCTSTR				text;					// Text

		int					top;					// OSD position: top
		int					left;					// OSD position: left
	} OSD_DISPLAY_PARM, *POSD_DISPLAY_PARM;

	COSDManager(HWND hParent, int countOSD);
	~COSDManager();
	
	// Hide all OSD
	void OSDAllHide();

	// Show OSD. return the OSD window size
	SIZE OSDShow(OSD_DISPLAY_PARM parm);
};

class CCaptionPainter {

	typedef struct _CAPTION_DISPLAY {
		CAPTION_DATA				data;			// caption data.
		LARGE_INTEGER				showTime;		// time to show do the caption data.
	} CAPTION_DISPLAY, *PCAPTION_DISPLAY;

	typedef std::list<CAPTION_DISPLAY> CAPTION_DISPLAY_LIST, *PCAPTION_DISPLAY_LIST;

	unsigned int			m_delayTime;
	HWND					m_hViewer;

	CCaptionDllUtil			m_captionUtil;			// CCaptionDllUtil for parsing caption packets.
	COSDManager *			m_pOSDManager;

	CAPTION_DISPLAY_LIST	m_showCaptionList;		// Stores caption datas.
	HANDLE					m_hNewCaptionEvent;		// Event that be singled when new caption data in.
	HANDLE					m_hListMutex;			// Mutex for m_showCaptionList.
	
	// Thread Proc for displaying captions.
	static unsigned int WINAPI CaptionDisplayProc(LPVOID lpParameter);

	HANDLE					m_hDisplayThreadHandle;	// Thread handle of CaptionDisplayProc
	HANDLE					m_hTerminateEvent;		// Event that be singled when CCaptionPainter is finalizing.

	void ShowCaptionData(CAPTION_DATA data);

public:
	CCaptionPainter(HWND hViewer);
	~CCaptionPainter();

	bool Initialize();

	// Clears all displayed captions.
	void Clear();

	// Input caption packets here.
	void AddCaptionPacket(unsigned char *pPacket);

	// Sets the delay time of caption display.
	void SetDelayTime(unsigned int msec);
};

#endif //_CCAPTION_PAINTER_H_